#install.packages("hqreg")
#install.packages("glmnet")
#install.packages("microbenchmark")
library(hqreg)
library(glmnet)
library(microbenchmark)

# data generation
# standardize the response to compare with glmnet (check help("glmnet") Details) 
simData <- function(n, p, SNR, dataSeed) {  
  b <- (-1)^(1:p)*exp(-(1:p-1)/10)
  b0 <- 10
  set.seed(dataSeed)
  X <- matrix(rnorm(n*p), nrow = n)
  U <- rnorm(n)
  X <- sweep(X, 1, 0.5*U, "+")
  Xb <- drop(X%*%b)
  s2 <- var(Xb)/SNR
  t <- sqrt(s2/2)
  Y <- b0 + Xb+t*rt(n, df = 4) # the error term t*rt(df=4) has variance var(Xb)/SNR
  Y <- Y/sd(Y)
  list(Y = Y,X = X)
}

# timing functions 
time_huber <- function(case_ids, nseq, pseq, gamma_seq, screen, times = 3) {
  t_tab <- matrix(nrow = length(case_ids), ncol = length(gamma_seq), 
                  dimnames = list(paste("Dataset", case_ids), paste("Gamma =", gamma_seq)))
  for(i in 1:length(case_ids)) {
    id <- case_ids[i]
    cat("Dataset", id, "\n")
    data <- simData(nseq[id],pseq[id], 3, id) # use case id as seed
    Y <- data$Y
    X <- data$X
    for (j in 1:length(gamma_seq)) {
      cat("Gamma =", gamma_seq[j], "\n")
      t <- microbenchmark(hqreg(X, Y, gamma = gamma_seq[j], alpha = 0.9, lambda.min = 0.05, screen = screen), times = times)$time
      t_tab[i,j] <- mean(t)*1e-9 # convert nanoseconds to seconds
    }
  }
  t_tab
}

time_quantile <- function(case_ids, nseq, pseq, tau_seq, screen, times = 3) {
  t_tab <- matrix(nrow = length(case_ids), ncol = length(tau_seq), 
                  dimnames = list(paste("Dataset", case_ids), paste("Tau =", tau_seq)))
  print(dim(t_tab))
  for(i in 1:length(case_ids)) {
    id <- case_ids[i]
    cat("Dataset", id, "\n")
    data <- simData(nseq[id],pseq[id], 3, id) # use case id as seed
    Y <- data$Y
    X <- data$X
    for (j in 1:length(tau_seq)) {
      cat("Tau =", tau_seq[j], "\n")
      t <- microbenchmark(hqreg(X, Y, method = "quantile", tau = tau_seq[j], alpha = 0.9, lambda.min = 0.05, screen = screen), times = times)$time
      t_tab[i,j] <- mean(t)*1e-9 # convert nanoseconds to seconds
    }
  }
  t_tab
}

time_ls <- function(case_ids, nseq, pseq, FUN, screen, times = 3) {
  t_seq <- double(length(case_ids))
  names(t_seq) <- paste("Dataset", case_ids)
  for(i in 1:length(case_ids)) {
    id <- case_ids[i]
    cat("Dataset", id, "\n")
    data <- simData(nseq[id],pseq[id], 3, id) # use case id as seed
    Y <- data$Y
    X <- data$X
    if (FUN == "hqreg") {
      t <- microbenchmark(hqreg(X, Y, method = "ls", alpha = 0.9, lambda.min = 0.05, screen = screen), times = times)$time
    } else if (FUN == "glmnet") {
      t <- microbenchmark(glmnet(X, Y, alpha = 0.9, lambda.min.ratio = 0.05), times = times)$time
    }
    t_seq[i] <- mean(t)*1e-9 # convert nanoseconds to seconds
  }
  t_seq
}

# two versions of hqreg: hqreg_ASR - adaptive strong rule; hqreg_NVS - no variable screening
# cases considered
nseq <- c(1000, 5000, rep(100, 4))
pseq <- c(100, 100, 1000, 5000, 20000, 100000)

# Huber loss regression
gamma_seq <- 10^(-2:2)
(t_hqreg_ASR_huber <- time_huber(1:6, nseq, pseq, gamma_seq, "ASR"))
(t_hqreg_NVS_huber <- time_huber(1:6, nseq, pseq, gamma_seq, "none"))

# Least squares
(t_glmnet_ls <- time_ls(1:6, nseq, pseq, "glmnet"))
(t_hqreg_ASR_ls <- time_ls(1:6, nseq, pseq, "hqreg", "ASR"))
(t_hqreg_NVS_ls <- time_ls(1:6, nseq, pseq, "hqreg", "none"))

# Quantile regression
tau_seq <- c(0.25, 0.5, 0.75)
# first 4 cases
(t_hqreg_ASR_quant <- time_quantile(1:4, nseq, pseq, tau_seq, "ASR"))
(t_hqreg_NVS_quant <- time_quantile(1:4, nseq, pseq, tau_seq, "none"))

# hqreg_NVS is shown to be much slower
# consider only hqreg_ASR for the last case
(t_hqreg_ASR_quant_last <- time_quantile(6, nseq, pseq, tau_seq, "ASR"))

